module Swagger
  module Api
    class ComponentSchema
      include ActiveAttr::Model
      include Concerns::Columns
      attr_accessor :controller

      def create
        {
          required: required,
          properties: properties,
          type: 'object'
        }
      end

      private

      def required
        controller.try(:columns).try(:required) ||
        columns.map(&:name) & clean_required_attributes
      end

      def clean_required_attributes
        required_attributes.map do |attribute|
          if model.reflect_on_all_associations(:belongs_to).map(&:name).map(&:to_s).include?(attribute)
            "#{attribute}_id"
          else
            attribute
          end
        end.compact.uniq.map(&:to_s)
      end

      def required_attributes
        model.validators.map do |validator|
          validator.attributes if validator.is_a?(ActiveRecord::Validations::PresenceValidator)
        end.compact.flatten.uniq.map(&:to_s)
      end

      def properties
        properties = {}
        columns.each do |column|
          properties[column.name] = ColumnSchema.new(column: column).create
        end
        properties
      end

      def model
        controller.model.constantize
      end
    end
  end
end