module Swagger
  module Api
    class Paths
      include ActiveAttr::Model
      attr_accessor :controllers

      def create
        return @paths unless @paths.nil?
        @paths = {}
        controllers.each do |controller|
          if controller.custom_path_file.nil?
            @paths.merge!(paths(controller))
          else
            @paths.merge!(custom_json(controller))
          end
        end
        @paths
      end

      private

      def paths(controller)
        paths = {}
        paths["#{route(controller)}"] = {}
        paths["#{route(controller)}{id}/"] = {}
        paths["#{route(controller)}"][:get] = Swagger::Api::Operations::Index.new(controller: controller).create if action?(controller, 'index')
        paths["#{route(controller)}"][:post] = Swagger::Api::Operations::Create.new(controller: controller).create if action?(controller, 'create')
        paths["#{route(controller)}{id}/"][:get] = Swagger::Api::Operations::Show.new(controller: controller).create if action?(controller, 'show')
        paths["#{route(controller)}{id}/"][:put] = Swagger::Api::Operations::Update.new(controller: controller).create if action?(controller, 'update')
        paths["#{route(controller)}{id}/"][:delete] = Swagger::Api::Operations::Delete.new(controller: controller).create if action?(controller, 'delete')
        paths["#{route(controller)}"].delete if paths["#{route(controller)}"].blank?
        paths["#{route(controller)}{id}/"].delete if paths["#{route(controller)}{id}/"].blank?
        paths
      end

      def route(controller)
        "/#{controller.name.demodulize.underscore.gsub('_controller', '')}/"
      end

      def custom_json(controller)
        file = File.read(controller.custom_path_file)
        JSON.parse(file)
      end

      def actions(controller)
        Swagger::Api::Actions.new(controller: controller).all!
      end

      def action?(controller, action_name)
        actions(controller).include?(action_name)
      end
    end
  end
end

